@@ -33,7 +33,8 @@
 -------------------------------------------------------------------------
     Contributing author and copyright for this file:
 
-    Christoph Kloss (DCS Computing GmbH, Linz, JKU Linz)
+    Christoph Kloss (DCS Computing GmbH, Linz)
+    Christoph Kloss (JKU Linz)
     Richard Berger (JKU Linz)
 
     Copyright 2012-     DCS Computing GmbH, Linz
@@ -60,12 +61,18 @@ namespace ContactModels {
   public:
     static const int MASK = CM_CONNECT_TO_PROPERTIES | CM_SURFACES_INTERSECT;
 
-    CohesionModel(LAMMPS * lmp, IContactHistorySetup*) : Pointers(lmp), cohEnergyDens(NULL)
+    int bond_history_offset() {return -1;}
+
+    CohesionModel(LAMMPS * lmp, IContactHistorySetup*,class ContactModelBase *) :
+        Pointers(lmp), cohEnergyDens(NULL)
     {
       
     }
 
-    void registerSettings(Settings&) {}
+    void registerSettings(Settings& settings) 
+    {
+        settings.registerOnOff("tangential_reduce",tangentialReduce_,false);
+    }
 
     void connectToProperties(PropertyRegistry & registry) {
       registry.registerProperty("cohEnergyDens", &MODEL_PARAMS::createCohesionEnergyDensity);
@@ -89,7 +96,7 @@ namespace ContactModels {
       else
         Acont = - M_PI/4 * ( (r-ri-rj)*(r+ri-rj)*(r-ri+rj)*(r+ri+rj) )/(r*r); //contact area of the two spheres
       const double Fn_coh = -cohEnergyDens[sidata.itype][sidata.jtype]*Acont;
-      sidata.Fn += Fn_coh;
+      if(tangentialReduce_) sidata.Fn += Fn_coh; 
 
       if(sidata.contact_flags) *sidata.contact_flags |= CONTACT_COHESION_MODEL;
 
@@ -125,6 +132,7 @@ namespace ContactModels {
 
   private:
     double ** cohEnergyDens;
+    bool tangentialReduce_;
   };
 }
 }
