@@ -33,7 +33,8 @@
 -------------------------------------------------------------------------
     Contributing author and copyright for this file:
 
-    Christoph Kloss (DCS Computing GmbH, Linz, JKU Linz)
+    Christoph Kloss (DCS Computing GmbH, Linz)
+    Christoph Kloss (JKU Linz)
     Richard Berger (JKU Linz)
     Alexander Podlozhnyuk (DCS Computing GmbH, Linz)
 
@@ -45,7 +46,7 @@
 #define CONTACT_INTERFACE_H_
 
 #include <string>
-#include "superquadric.h"
+#include "superquadric_flag.h"
 
 namespace LIGGGHTS {
 namespace ContactModels {
@@ -77,8 +78,14 @@ struct SurfacesCloseData {
   double *shape_j; //shape parameters of j-th particle (a,b,c)
   double *roundness_i; //roundness parameters of i-th particle (eps1, eps2)
   double *roundness_j; //roundness parameters of j-th particle (eps1, eps2)
+  double contact_point[3];
   double *pos_i;
   double *pos_j;
+  double *inertia_i;
+  double *inertia_j;
+  bool is_non_spherical;
+  double koefi;
+  double koefj;
   SurfacesCloseData() : area_ratio(1.0),
                         quat_i(NULL),
                         quat_j(NULL),
@@ -87,7 +94,12 @@ struct SurfacesCloseData {
                         roundness_i(NULL),
                         roundness_j(NULL),
                         pos_i(NULL),
-                        pos_j(NULL){}
+                        pos_j(NULL),
+                        inertia_i(NULL),
+                        inertia_j(NULL),
+                        is_non_spherical(false),
+                        koefi(0.0),
+                        koefj(0.0) {}
 #else
   SurfacesCloseData() : area_ratio(1.0) {}
 #endif
@@ -128,6 +140,8 @@ struct SurfacesIntersectData : SurfacesCloseData {
   double mj;
   double meff;
 
+  mutable double P_diss; 
+
   int computeflag;
   int shearupdate;
   int itype;
