@@ -72,7 +72,9 @@ FixContactHistoryMesh::FixContactHistoryMesh(LAMMPS *lmp, int narg, char **arg)
   ipage2_(0),
   dpage2_(0),
   keeppage_(0),
+  intersectpage_(0),
   keepflag_(0),
+  intersectflag_(0),
   mesh_(0),
   fix_neighlist_mesh_(0),
   fix_nneighs_(0),
@@ -89,9 +91,12 @@ FixContactHistoryMesh::FixContactHistoryMesh(LAMMPS *lmp, int narg, char **arg)
 
   swap_ = new double[dnum_];
 
-  // initial allocation of delflag
+  // initial allocation of keepflag
   keepflag_ = (bool **) memory->srealloc(keepflag_,atom->nmax*sizeof(bool *),
                                       "contact_history:keepflag");
+  // initial allocation of intersectflag
+  intersectflag_ = (bool **) memory->srealloc(intersectflag_,atom->nmax*sizeof(bool *),
+                                      "contact_history:intersectflag");
 }
 
 /* ---------------------------------------------------------------------- */
@@ -113,6 +118,14 @@ FixContactHistoryMesh::~FixContactHistoryMesh()
     delete [] keeppage_;
     keeppage_ = NULL;
   }
+  if(intersectpage_) {
+    for(int i = 0; i < numpages_; i++) {
+      delete intersectpage_[i];
+      intersectpage_[i] = NULL;
+    }
+    delete [] intersectpage_;
+    intersectpage_ = NULL;
+  }
 
   ipage_ = 0;
   dpage_ = 0;
@@ -120,6 +133,7 @@ FixContactHistoryMesh::~FixContactHistoryMesh()
   delete [] swap_;
 
   if(keepflag_) memory->sfree(keepflag_);
+  if(intersectflag_) memory->sfree(intersectflag_);
 }
 
 /* ---------------------------------------------------------------------- */
@@ -173,6 +187,14 @@ void FixContactHistoryMesh::allocate_pages()
       delete [] keeppage_;
       keeppage_ = NULL;
     }
+    if(intersectpage_) {
+      for(int i = 0; i < numpages_; i++) {
+        delete intersectpage_[i];
+        intersectpage_[i] = NULL;
+      }
+      delete [] intersectpage_;
+      intersectpage_ = NULL;
+    }
 
     pgsize_ = neighbor->pgsize;
     oneatom_ = neighbor->oneatom;
@@ -182,6 +204,7 @@ void FixContactHistoryMesh::allocate_pages()
     ipage2_ = new MyPage<int>[numpages_];
     dpage2_ = new MyPage<double>[numpages_];
     keeppage_ = new MyPage<bool>*[numpages_];
+    intersectpage_ = new MyPage<bool>*[numpages_];
     for (int i = 0; i < numpages_; i++) {
       ipage1_[i].init(oneatom_,pgsize_);
       dpage1_[i].init(oneatom_*MathExtraLiggghts::max(1,dnum_),pgsize_);
@@ -196,11 +219,15 @@ void FixContactHistoryMesh::allocate_pages()
       // make sure page is allocated in memory near core
       keeppage_[tid] = new MyPage<bool>();
       keeppage_[tid]->init(oneatom_,pgsize_);
+      intersecthpage_[tid] = new MyPage<bool>();
+      intersectpage_[tid]->init(oneatom_,pgsize_);
     }
 #else
     for (int i = 0; i < numpages_; i++) {
       keeppage_[i] = new MyPage<bool>();
       keeppage_[i]->init(oneatom_,pgsize_);
+      intersectpage_[i] = new MyPage<bool>();
+      intersectpage_[i]->init(oneatom_,pgsize_);
     }
 #endif
 
@@ -290,10 +317,15 @@ void FixContactHistoryMesh::pre_force(int dummy)
         return;
     build_neighlist_ = false;
 
+    int nall = atom->nlocal+atom->nghost;
+    int nlocal = atom->nlocal;
+
+    for(int i = nlocal; i < nall; i++)
+        npartner_[i] = 0;
+
     cleanUpContactJumps();
 
     int nneighs_next;
-    int nlocal = atom->nlocal;
     int *partner_prev;
     double *contacthistory_prev;
 
@@ -303,7 +335,7 @@ void FixContactHistoryMesh::pre_force(int dummy)
     ipage_next->reset();
     dpage_next->reset();
 
-    for (int i = 0; i < nlocal; i++)
+    for (int i = 0; i < nall; i++)
     {
         nneighs_next = fix_nneighs_->get_vector_atom_int(i);
 
@@ -376,19 +408,21 @@ void FixContactHistoryMesh::sort_contacts()
 }
 
 /* ----------------------------------------------------------------------
-     mark all contacts for deletion
+     mark all contacts for deletion, mark all as not intersecting
 ------------------------------------------------------------------------- */
 
 void FixContactHistoryMesh::markAllContacts()
 {
-    int nlocal = atom->nlocal;
+    int nall = atom->nlocal+atom->nghost;
     keeppage_[0]->reset(true);
+    intersectpage_[0]->reset(false);
 
-    for(int i = 0; i < nlocal; i++)
+    for(int i = 0; i < nall; i++)
     {
       const int nneighs = fix_nneighs_->get_vector_atom_int(i);
       keepflag_[i] = keeppage_[0]->get(nneighs);
-      if (!keepflag_[i])
+      intersectflag_[i] = intersectpage_[0]->get(nneighs);
+      if (!keepflag_[i] || !intersectflag_[i])
         error->one(FLERR,"mesh contact history overflow, boost neigh_modify one");
     }
 }
@@ -401,6 +435,7 @@ void FixContactHistoryMesh::resetDeletionPage(int tid)
 {
     // keep pages are initalized with 0 (= false)
     keeppage_[tid]->reset(true);
+    intersectpage_[tid]->reset(false);
 }
 
 /* ----------------------------------------------------------------------
@@ -413,7 +448,8 @@ void FixContactHistoryMesh::markForDeletion(int tid, int ifrom, int ito)
     {
       const int nneighs = fix_nneighs_->get_vector_atom_int(i);
       keepflag_[i] = keeppage_[tid]->get(nneighs);
-      if (!keepflag_[i])
+      intersectflag_[i] = intersectpage_[tid]->get(nneighs);
+      if (!keepflag_[i] || !intersectflag_[i])
         error->one(FLERR,"mesh contact history overflow, boost neigh_modify one");
     }
 }
@@ -422,7 +458,7 @@ void FixContactHistoryMesh::markForDeletion(int tid, int ifrom, int ito)
 
 void FixContactHistoryMesh::cleanUpContacts()
 {
-    cleanUpContacts(0, atom->nlocal);
+    cleanUpContacts(0, atom->nlocal+atom->nghost);
 }
 
 /* ---------------------------------------------------------------------- */
@@ -446,6 +482,7 @@ void FixContactHistoryMesh::cleanUpContacts(int ifrom, int ito)
                     
                 }
                 partner_[i][j] = -1;
+                intersectflag_[i][j] = false;
                 vectorZeroizeN(&(contacthistory_[i][j*dnum_]),dnum_);
             }
         }
@@ -457,10 +494,10 @@ void FixContactHistoryMesh::cleanUpContacts(int ifrom, int ito)
 
 void FixContactHistoryMesh::cleanUpContactJumps()
 {
-    int nlocal = atom->nlocal;
+    int nall = atom->nlocal+atom->nghost;
     int iTri;
 
-    for(int i = 0; i < nlocal; i++)
+    for(int i = 0; i < nall; i++)
     {
         int ipartner = 0;
         while (ipartner < npartner_[i])
@@ -478,6 +515,7 @@ void FixContactHistoryMesh::cleanUpContactJumps()
             {
                 
                 partner_[i][ipartner] = -1;
+                intersectflag_[i][ipartner] = false;
                 vectorZeroizeN(&(contacthistory_[i][ipartner*dnum_]),dnum_);
                 swap(i,ipartner,npartner_[i]-1,false);
                 npartner_[i]--;
@@ -493,9 +531,9 @@ void FixContactHistoryMesh::cleanUpContactJumps()
 
 void FixContactHistoryMesh::reset_history()
 {
-    int nlocal = atom->nlocal;
+    int nall = atom->nlocal+atom->nghost;
 
-    for(int i = 0; i < nlocal; i++)
+    for(int i = 0; i < nall; i++)
     {
         const int nneighs = fix_nneighs_->get_vector_atom_int(i);
 
@@ -529,6 +567,8 @@ void FixContactHistoryMesh::grow_arrays(int nmax)
   FixContactHistory::grow_arrays(nmax);
   keepflag_ = (bool **) memory->srealloc(keepflag_,nmax*sizeof(bool *),
                                       "contact_history:keepflag");
+  intersectflag_ = (bool **) memory->srealloc(intersectflag_,nmax*sizeof(bool *),
+                                      "contact_history:keepflag");
 }
 
 /* ----------------------------------------------------------------------
@@ -545,6 +585,7 @@ void FixContactHistoryMesh::copy_arrays(int i, int j, int delflag)
 
   FixContactHistory::copy_arrays(i,j,delflag);
   keepflag_[j] = keepflag_[i];
+  intersectflag_[j] = intersectflag_[i];
 }
 
 /* ----------------------------------------------------------------------
@@ -655,6 +696,7 @@ double FixContactHistoryMesh::memory_usage()
     bytes += ipage2_[i].size();
     bytes += dpage2_[i].size();
     bytes += keeppage_[i]->size();
+    bytes += intersectpage_[i]->size();
   }
 
   return bytes;
