@@ -32,11 +32,11 @@
 
 -------------------------------------------------------------------------
     Contributing author and copyright for this file:
-    (if not contributing author is listed, this file has been contributed
-    by the core developer)
+    Christoph Kloss (JKU Linz, DCS Computing GmbH, Linz)
+    Richard Berger (JKU Linz)
 
     Copyright 2012-     DCS Computing GmbH, Linz
-    Copyright 2009-2012 JKU Linz
+    Copyright 2009-2015 JKU Linz
 ------------------------------------------------------------------------- */
 
 #include "math.h"
@@ -61,6 +61,10 @@
 #include "mpi_liggghts.h"
 #include "vector_liggghts.h"
 
+#include "probability_distribution.h"
+#include "region_neighbor_list.h"
+#include "superquadric_flag.h"
+
 using namespace LAMMPS_NS;
 using namespace FixConst;
 
@@ -72,7 +76,8 @@ using namespace FixConst;
 /* ---------------------------------------------------------------------- */
 
 FixInsert::FixInsert(LAMMPS *lmp, int narg, char **arg) :
-  Fix(lmp, narg, arg)
+  Fix(lmp, narg, arg),
+  neighList(*new RegionNeighborList<interpolate_no>(lmp))
 {
   if (narg < 7) error->fix_error(FLERR,this,"not enough arguments");
 
@@ -86,6 +91,8 @@ FixInsert::FixInsert(LAMMPS *lmp, int narg, char **arg) :
   fix_multisphere = NULL;
   multisphere = NULL;
 
+  compress_flag = false ;
+
   // required args
   iarg = 3;
 
@@ -99,18 +106,21 @@ FixInsert::FixInsert(LAMMPS *lmp, int narg, char **arg) :
   // set defaults
   init_defaults();
 
-  xnear = NULL;
-
   // parse args
   
+#ifdef SUPERQUADRIC_ACTIVE_FLAG
+  check_obb_flag = 1;
+#endif
+  
   bool hasargs = true;
   while(iarg < narg && hasargs)
   {
     hasargs = false;
+    
     if(strcmp(arg[iarg],"distributiontemplate") == 0) {
       if (iarg+2 > narg) error->fix_error(FLERR,this,"");
       int ifix = modify->find_fix(arg[iarg+1]);
-      if(ifix < 0 || strcmp(modify->fix[ifix]->style,"particledistribution/discrete"))
+      if(ifix < 0 || strncmp(modify->fix[ifix]->style,"particledistribution/discrete",29))
         error->fix_error(FLERR,this,"Fix insert requires you to define a valid ID for a fix of type particledistribution/discrete");
       fix_distribution = static_cast<FixParticledistributionDiscrete*>(modify->fix[ifix]);
       iarg += 2;
@@ -194,6 +204,16 @@ FixInsert::FixInsert(LAMMPS *lmp, int narg, char **arg) :
       else error->fix_error(FLERR,this,"");
       iarg += 2;
       hasargs = true;
+    } else if (strcmp(arg[iarg],"compress_tags") == 0) {
+      if (iarg+2 > narg) error->fix_error(FLERR,this,"not enough arguments for compress_tags");
+      if(0 == strcmp(arg[iarg+1],"yes"))
+        compress_flag = true;
+      else if(0 == strcmp(arg[iarg+1],"no"))
+        compress_flag = false;
+      else
+        error->fix_error(FLERR,this,"expecting 'yes' or 'no' after 'compress_tags'");
+      iarg += 2;
+      hasargs = true;
     } else if (strcmp(arg[iarg],"vel") == 0) {
       if (iarg+5 > narg) error->fix_error(FLERR,this,"not enough keyword for 'vel'");
       if (strcmp(arg[iarg+1],"constant") == 0)  {
@@ -203,7 +223,7 @@ FixInsert::FixInsert(LAMMPS *lmp, int narg, char **arg) :
           iarg += 5;
       } else if (strcmp(arg[iarg+1],"uniform") == 0) {
           if (iarg+8 > narg) error->fix_error(FLERR,this,"not enough keyword for 'uniform'");
-          v_randomSetting = 1; //switch 1...distribute with equal prop.
+          v_randomSetting = RANDOM_UNIFORM;
           v_insert[0] = atof(arg[iarg+2]);
           v_insert[1] = atof(arg[iarg+3]);
           v_insert[2] = atof(arg[iarg+4]);
@@ -213,7 +233,7 @@ FixInsert::FixInsert(LAMMPS *lmp, int narg, char **arg) :
           iarg += 8;
       } else if (strcmp(arg[iarg+1],"gaussian") == 0) {
           if (iarg+8 > narg) error->fix_error(FLERR,this,"not enough keyword for 'gaussian'");
-          v_randomSetting = 2; //switch 2...distribute with gaussian distrib.
+          v_randomSetting = RANDOM_GAUSSIAN;
           v_insert[0] = atof(arg[iarg+2]);
           v_insert[1] = atof(arg[iarg+3]);
           v_insert[2] = atof(arg[iarg+4]);
@@ -260,6 +280,17 @@ FixInsert::FixInsert(LAMMPS *lmp, int narg, char **arg) :
       hasargs = true;
     }
     
+#ifdef SUPERQUADRIC_ACTIVE_FLAG
+    else if (strcmp(arg[iarg],"check_obb") == 0) {
+      if (iarg+2 > narg) error->fix_error(FLERR,this,"");
+      if(strcmp(arg[iarg+1],"yes")==0) check_obb_flag = 1;
+      else if(strcmp(arg[iarg+1],"no")==0) check_obb_flag = 0;
+      else error->fix_error(FLERR,this,"");
+      if(check_ol_flag==0) check_obb_flag = 0;
+      iarg += 2;
+      hasargs = true;
+    }
+#endif
     else if(strcmp(style,"insert") == 0) error->fix_error(FLERR,this,"unknown keyword");
   }
 
@@ -308,6 +339,7 @@ FixInsert::~FixInsert()
   delete random;
   delete [] recvcounts;
   delete [] displs;
+  delete &neighList;
   if(property_name) delete []property_name;
 }
 
@@ -371,7 +403,7 @@ void FixInsert::init_defaults()
 
   exact_number = 1;
 
-  v_randomSetting = 0;
+  v_randomSetting = RANDOM_CONSTANT;
   vectorZeroize3D(v_insert);
   vectorZeroize3D(v_insertFluct);
   vectorZeroize3D(omega_insert);
@@ -415,23 +447,23 @@ void FixInsert::print_stats_start()
     if(ninsert_exists)
     {
         if (screen)
-            fprintf(screen ,"INFO: Particle insertion %s: %f particles every %d steps - particle rate %f  (mass rate %f)\n"
-                            "      %d particles (mass %f) within %d steps\n",
+            fprintf(screen ,"INFO: Particle insertion %s: %f particles every %d steps - particle rate %f  (mass rate %e)\n"
+                            "      %d particles (mass %e) within %d steps\n",
                 id,ninsert_per,insert_every,nflowrate,massflowrate,ninsert,massinsert,final_ins_step-first_ins_step);
 
         if (logfile)
-            fprintf(logfile,"INFO: Particle insertion %s: %f particles every %d steps - particle rate %f, (mass rate %f)\n"
-                            "      %d particles (mass %f) within %d steps\n",
+            fprintf(logfile,"INFO: Particle insertion %s: %f particles every %d steps - particle rate %f, (mass rate %e)\n"
+                            "      %d particles (mass %e) within %d steps\n",
                 id,ninsert_per,insert_every,nflowrate,massflowrate,ninsert,massinsert,final_ins_step-first_ins_step);
     }
     else if(massflowrate > 0.)
     {
         if (screen)
-            fprintf(screen ,"INFO: Particle insertion %s: %f particles every %d steps - particle rate %f  (mass rate %f)\n",
+            fprintf(screen ,"INFO: Particle insertion %s: %f particles every %d steps - particle rate %f  (mass rate %e)\n",
                 id,ninsert_per,insert_every,nflowrate,massflowrate);
 
         if (logfile)
-            fprintf(logfile,"INFO: Particle insertion %s: %f particles every %d steps - particle rate %f, (mass rate %f)\n",
+            fprintf(logfile,"INFO: Particle insertion %s: %f particles every %d steps - particle rate %f, (mass rate %e)\n",
                 id,ninsert_per,insert_every,nflowrate,massflowrate);
     }
     else
@@ -454,13 +486,13 @@ void FixInsert::print_stats_during(int ninsert_this, double mass_inserted_this)
   if (me == 0 && print_stats_during_flag)
   {
     if (screen)
-      fprintf(screen ,"INFO: Particle insertion %s: inserted %d particle templates (mass %f) at step "BIGINT_FORMAT"\n"
-                      " - a total of %d particle templates (mass %f) inserted so far.\n",
+      fprintf(screen ,"INFO: Particle insertion %s: inserted %d particle templates (mass %e) at step " BIGINT_FORMAT "\n"
+                      " - a total of %d particle templates (mass %e) inserted so far.\n",
               id,ninsert_this,mass_inserted_this,step,ninserted,massinserted);
 
     if (logfile)
-      fprintf(logfile,"INFO: Particle insertion %s: inserted %d particle templates (mass %f) at step "BIGINT_FORMAT"\n"
-                      " - a total of %d particle templates (mass %f) inserted so far.\n",
+      fprintf(logfile,"INFO: Particle insertion %s: inserted %d particle templates (mass %e) at step " BIGINT_FORMAT "\n"
+                      " - a total of %d particle templates (mass %e) inserted so far.\n",
               id,ninsert_this,mass_inserted_this,step,ninserted,massinserted);
   }
 }
@@ -576,7 +608,9 @@ void FixInsert::pre_exchange()
   most_recent_ins_step = update->ntimestep;
 
   // things to be done before inserting new particles
-  pre_insert();
+  
+  if(!pre_insert())
+    return;
 
   // number of particles to insert this timestep
   ninsert_this = calc_ninsert_this();
@@ -650,10 +684,10 @@ void FixInsert::pre_exchange()
   // fill xnear array with particles to check overlap against
   
   // add particles in insertion volume to xnear list
-  nspheres_near = 0;
-  xnear = NULL;
+  neighList.reset();
+
   if(check_ol_flag)
-      nspheres_near = load_xnear(ninsert_this_local);
+    load_xnear(ninsert_this_local);
 
   // insertion counters in this step
   int ninserted_this = 0, ninserted_spheres_this = 0;
@@ -669,7 +703,7 @@ void FixInsert::pre_exchange()
   // actual particle insertion
 
   fix_distribution->pre_insert(ninserted_this_local,fix_property,fix_property_value);
-
+  
   ninserted_spheres_this_local = fix_distribution->insert(ninserted_this_local);
 
   // warn if max # insertions exceeded by random processes
@@ -681,9 +715,21 @@ void FixInsert::pre_exchange()
   // set tag # of new particles beyond all previous atoms, reset global natoms
   // if global map exists, reset it now instead of waiting for comm
   // since deleting atoms messes up ghosts
+  int step = update->ntimestep;
 
   if (atom->tag_enable)
   {
+
+    //force all tags to be reset by setting them to zero
+    if(compress_flag)
+    {
+        if(comm->me == 0)
+            printf("FixInsertStream: resetting tags @ step %d. \n", step);
+        int *tag = atom->tag;
+        for (int i = 0; i < atom->nlocal; i++)
+            tag[i] = 0;
+    }
+
     atom->tag_extend();
     atom->natoms += static_cast<double>(ninserted_spheres_this);
     if (atom->map_style)
@@ -712,9 +758,6 @@ void FixInsert::pre_exchange()
   if(ninserted_this < ninsert_this && comm->me == 0)
       error->warning(FLERR,"Particle insertion: Less insertions than requested");
 
-  // free local memory
-  if(xnear) memory->destroy(xnear);
-
   // next timestep to insert
   if (insert_every && (!ninsert_exists || ninserted < ninsert)) next_reneighbor += insert_every;
   else next_reneighbor = 0;
@@ -825,42 +868,61 @@ int FixInsert::count_nnear()
 }
 
 /* ----------------------------------------------------------------------
-   fill xnear with nearby particles
+   fill neighbor list with nearby particles
 ------------------------------------------------------------------------- */
 
 int FixInsert::load_xnear(int ninsert_this_local)
 {
-  // count nearby spheres
-  // setup for allgatherv
-  int nspheres_near_local = count_nnear();
-
-  // data size per particle: x and radius
-  //int n = 4*nspheres_near;
-
-  // xnear is for my atoms + atoms to be inserted
-  
-  memory->create(xnear,nspheres_near_local + ninsert_this_local*fix_distribution->max_nspheres(), 4, "FixInsert::xnear");
-
-  // load up xnear array with local and ghosts
+  // load up neighbor list with local and ghosts
 
   double **x = atom->x;
   double *radius = atom->radius;
-  int nall = atom->nlocal + atom->nghost;
+  const int nall = atom->nlocal + atom->nghost;
 
-  int ncount = 0;
-  for (int i = 0; i < nall; i++)
+  BoundingBox bb = getBoundingBox();
+  neighList.reset();
+#ifdef SUPERQUADRIC_ACTIVE_FLAG
+  neighList.set_obb_flag(check_obb_flag);
+#endif
+
+  if(neighList.setBoundingBox(bb, maxrad))
   {
-    if (is_nearby(i))
+    for (int i = 0; i < nall; ++i)
     {
-      xnear[ncount][0] = x[i][0];
-      xnear[ncount][1] = x[i][1];
-      xnear[ncount][2] = x[i][2];
-      xnear[ncount][3] = radius[i];
-      ncount++;
+      if (is_nearby(i))
+      {
+#ifdef SUPERQUADRIC_ACTIVE_FLAG
+        if(atom->superquadric_flag and check_obb_flag)
+          neighList.insert_superquadric(x[i], radius[i], atom->quaternion[i], atom->shape[i]);
+        else
+          neighList.insert(x[i], radius[i]);
+#else
+        neighList.insert(x[i], radius[i]);
+#endif
+      }
     }
   }
 
-  return nspheres_near_local;
+  return neighList.count();
+}
+
+/* ----------------------------------------------------------------------
+   generate random velocity based on random setting
+------------------------------------------------------------------------- */
+
+void FixInsert::generate_random_velocity(double * velocity) {
+  switch(v_randomSetting) {
+    case RANDOM_UNIFORM:
+      velocity[0] = v_insert[0] + v_insertFluct[0] * 2.0 * (random->uniform()-0.50);
+      velocity[1] = v_insert[1] + v_insertFluct[1] * 2.0 * (random->uniform()-0.50);
+      velocity[2] = v_insert[2] + v_insertFluct[2] * 2.0 * (random->uniform()-0.50);
+      break;
+
+    case RANDOM_GAUSSIAN:
+      velocity[0] = v_insert[0] + v_insertFluct[0] * random->gaussian();
+      velocity[1] = v_insert[1] + v_insertFluct[1] * random->gaussian();
+      velocity[2] = v_insert[2] + v_insertFluct[2] * random->gaussian();
+  }
 }
 
 /* ----------------------------------------------------------------------
