@@ -33,7 +33,8 @@
 -------------------------------------------------------------------------
     Contributing author and copyright for this file:
 
-    Christoph Kloss (DCS Computing GmbH, Linz, JKU Linz)
+    Christoph Kloss (DCS Computing GmbH, Linz)
+    Christoph Kloss (JKU Linz)
     Philippe Seil (JKU Linz)
 
     Copyright 2012-     DCS Computing GmbH, Linz
@@ -68,6 +69,10 @@ namespace LAMMPS_NS
 
         void setPrecision(double _precision);
 
+        void setMinFeatureLength(double _min_feature_length);
+
+        void setElementExclusionList(FILE *_file);
+
         void autoRemoveDuplicates();
 
         // scale mesh
@@ -204,6 +209,8 @@ namespace LAMMPS_NS
         // mesh ID - same as fix mesh ID
         char *mesh_id_;
 
+        // for overlap check on element insertion
+        
         inline void reset_stepLastReset()
         { stepLastReset_ = -1; }
 
@@ -215,11 +222,22 @@ namespace LAMMPS_NS
         inline double precision()
         { return precision_; }
 
+        inline double minFeatureLength()
+        { return min_feature_length_; }
+
+        inline FILE* elementExclusionList()
+        { return element_exclusion_list_; }
+
       private:
 
         // mesh precision
         double precision_;
 
+        // ignore features smaller than this size
+        double min_feature_length_;
+
+        FILE *element_exclusion_list_;
+
         // state if elements should be automatically removed if duplicate
         bool autoRemoveDuplicates_;
 
