@@ -51,6 +51,7 @@
 #include "mpi.h"
 #include "string.h"
 #include "stdlib.h"
+#include "dirent.h"
 #include "read_dump.h"
 #include "reader.h"
 #include "style_reader.h"
@@ -72,7 +73,7 @@ using namespace LAMMPS_NS;
 
 // also in reader_native.cpp
 
-enum{ID,TYPE,X,Y,Z,VX,VY,VZ,Q,IX,IY,IZ};
+enum{ID,TYPE,X,Y,Z,VX,VY,VZ,Q,IX,IY,IZ,RADIUS,MASS,DENSITY};
 enum{UNSET,NOSCALE_NOWRAP,NOSCALE_WRAP,SCALE_NOWRAP,SCALE_WRAP};
 
 /* ---------------------------------------------------------------------- */
@@ -201,6 +202,13 @@ void ReadDump::command(int narg, char **arg)
 
 void ReadDump::store_files(int nstr, char **str)
 {
+  
+  if(strrchr(str[0],'*'))
+  {
+    file_search(str[0]);
+    return;
+  }
+
   nfile = nstr;
   files = new char*[nfile];
 
@@ -211,6 +219,117 @@ void ReadDump::store_files(int nstr, char **str)
   }
 }
 
+/* ----------------------------------------------------------------------
+   infile contains a "*"
+   search for all files which match the infile pattern
+   replace "*" with any timestep value
+   search dir referenced by initial pathname of file
+------------------------------------------------------------------------- */
+
+void ReadDump::file_search(char *infile)
+{
+  char *ptr;
+
+  files = new char*[10000];
+  int *middle_index = new int[10000];
+
+  // separate infile into dir + filename
+
+  char *dirname = new char[strlen(infile) + 1];
+  char *filename = new char[strlen(infile) + 1];
+
+  if (strchr(infile,'/')) {
+    ptr = strrchr(infile,'/');
+    *ptr = '\0';
+    strcpy(dirname,infile);
+    strcpy(filename,ptr+1);
+    *ptr = '/';
+  } else {
+    strcpy(dirname,"./");
+    strcpy(filename,infile);
+  }
+
+  char *pattern = new char[strlen(filename) + 1];
+  strcpy(pattern,filename);
+
+  // scan all files in directory, searching for files that match pattern
+  // maxnum = largest int that matches "*"
+
+  size_t n = strlen(pattern) + 16; 
+  char *begin = new char[n];
+  char *middle = new char[n];
+  char *end = new char[n];
+
+  ptr = strchr(pattern,'*');
+  *ptr = '\0';
+  strcpy(begin,pattern);
+  strcpy(end,ptr+1);
+  int nbegin = strlen(begin);
+  nfile = 0;
+
+  struct dirent *ep;
+  DIR *dp = opendir(dirname);
+  if (dp == NULL)
+    error->one(FLERR,"Cannot open dir to search for dump file");
+
+  while ((ep = readdir(dp))) { 
+    if (strstr(ep->d_name,begin) != ep->d_name) continue;
+    if ((ptr = strstr(&ep->d_name[nbegin],end)) == NULL) continue;
+    if (strlen(end) == 0) ptr = ep->d_name + strlen(ep->d_name);
+    *ptr = '\0';
+    if (strlen(&ep->d_name[nbegin]) < n) {
+      strcpy(middle,&ep->d_name[nbegin]);
+
+      nfile++;
+
+      if(nfile >= 10000)
+        error->one(FLERR,"Currently max. 10000 dump files matching pattern can be read");
+
+      files[nfile-1] = new char[strlen(filename) + 16];
+      middle_index[nfile-1] = atoi(middle);
+      sprintf(files[nfile-1],"%s/%s%s%s",dirname,begin,middle,end);
+      
+    }
+  }
+  closedir(dp);
+  if (nfile <= 0) error->one(FLERR,"Found no dump file matching pattern");
+
+  bool swaped;
+  int nswaps_left = nfile;
+  do
+  {
+      swaped = false;
+      for(int i = 0; i < nfile-1; i++)
+      {
+          if(middle_index[i] > middle_index[i+1])
+          {
+            //swap
+            char swapper[512];
+            strcpy(swapper,files[i+1]);
+            strcpy(files[i+1],files[i]);
+            strcpy(files[i],swapper);
+
+            int swapper_i = middle_index[i+1];
+            middle_index[i+1] = middle_index[i];
+            middle_index[i] = swapper_i;
+
+            swaped = true;
+          }
+      }
+      nswaps_left--;
+  } while(swaped && nswaps_left > 0);
+
+  // clean up
+
+  delete [] dirname;
+  delete [] filename;
+  delete [] pattern;
+  delete [] begin;
+  delete [] middle;
+  delete [] end;
+  delete [] middle_index;
+}
+
 /* ---------------------------------------------------------------------- */
 
 void ReadDump::setup_reader(int narg, char **arg)
@@ -609,7 +728,7 @@ int ReadDump::fields_and_keywords(int narg, char **arg)
 
   nfield = 0;
   fieldtype[nfield++] = ID;
-  if (iarg < narg) fieldtype[nfield++] = TYPE;
+  /*if (iarg < narg)*/ fieldtype[nfield++] = TYPE; 
 
   // parse fields
 
@@ -626,6 +745,21 @@ int ReadDump::fields_and_keywords(int narg, char **arg)
         error->all(FLERR,"Read dump of atom property that isn't allocated");
       fieldtype[nfield++] = Q;
     }
+    else if (strcmp(arg[iarg],"radius") == 0) {
+      if (!atom->radius_flag)
+        error->all(FLERR,"Read dump of atom property that isn't allocated");
+      fieldtype[nfield++] = RADIUS;
+    }
+    else if (strcmp(arg[iarg],"mass") == 0) {
+      if (!atom->rmass_flag)
+        error->all(FLERR,"Read dump of atom property that isn't allocated");
+      fieldtype[nfield++] = MASS;
+    }
+    else if (strcmp(arg[iarg],"density") == 0) {
+      if (!atom->density_flag)
+        error->all(FLERR,"Read dump of atom property that isn't allocated");
+      fieldtype[nfield++] = DENSITY;
+    }
     else if (strcmp(arg[iarg],"ix") == 0) fieldtype[nfield++] = IX;
     else if (strcmp(arg[iarg],"iy") == 0) fieldtype[nfield++] = IY;
     else if (strcmp(arg[iarg],"iz") == 0) fieldtype[nfield++] = IZ;
@@ -748,6 +882,9 @@ void ReadDump::process_atoms(int n)
   double **x = atom->x;
   double **v = atom->v;
   double *q = atom->q;
+  double *radius = atom->radius;
+  double *rmass= atom->rmass;
+  double *density= atom->density;
   tagint *image = atom->image;
   int nlocal = atom->nlocal;
   int map_tag_max = atom->map_tag_max;
@@ -792,15 +929,24 @@ void ReadDump::process_atoms(int n)
         case VX:
           v[m][0] = fields[i][ifield];
           break;
-        case Q:
-          q[m] = fields[i][ifield];
-          break;
         case VY:
           v[m][1] = fields[i][ifield];
           break;
         case VZ:
           v[m][2] = fields[i][ifield];
           break;
+        case Q:
+          q[m] = fields[i][ifield];
+          break;
+        case RADIUS:
+          radius[m] = fields[i][ifield];
+          break;
+        case MASS:
+          rmass[m] = fields[i][ifield];
+          break;
+        case DENSITY:
+          density[m] = fields[i][ifield];
+          break;
         case IX:
           xbox = static_cast<int> (fields[i][ifield]);
           break;
@@ -873,6 +1019,9 @@ void ReadDump::process_atoms(int n)
 
     v = atom->v;
     q = atom->q;
+    radius = atom->radius;
+    rmass = atom->rmass;
+    density = atom->density;
     image = atom->image;
 
     // set atom attributes from other dump file fields
@@ -893,6 +1042,15 @@ void ReadDump::process_atoms(int n)
       case Q:
         q[m] = fields[i][ifield];
         break;
+      case RADIUS:
+        radius[m] = fields[i][ifield];
+        break;
+      case MASS:
+        rmass[m] = fields[i][ifield];
+        break;
+      case DENSITY:
+        density[m] = fields[i][ifield];
+        break;
       case IX:
         xbox = static_cast<int> (fields[i][ifield]);
         break;
