@@ -7,9 +7,11 @@
 #include "fix_ave_histo.h"
 #include "fix_ave_spatial.h"
 #include "fix_ave_time.h"
+#include "fix_base_liggghts.h"
 #include "fix_box_relax.h"
 #include "fix_buoyancy.h"
 #include "fix_cfd_coupling_convection.h"
+#include "fix_cfd_coupling_convection_impl.h"
 #include "fix_cfd_coupling_convection_species.h"
 #include "fix_cfd_coupling_force.h"
 #include "fix_cfd_coupling_force_implicit.h"
@@ -57,6 +59,7 @@
 #include "fix_planeforce.h"
 #include "fix_print.h"
 #include "fix_property_atom.h"
+#include "fix_property_atom_timetracer.h"
 #include "fix_property_atom_tracer.h"
 #include "fix_property_atom_tracer_stream.h"
 #include "fix_property_global.h"
